searchData={"items":[{"type":"module","title":"MSSMT","doc":"Merkle Sum Sparse Merkle Tree (MSSMT) implementation.\n\nProvides functions to create, insert, retrieve, delete, and verify elements in the MSSMT.","ref":"MSSMT.html"},{"type":"function","title":"MSSMT.delete/2","doc":"Deletes a key from the MSSMT.","ref":"MSSMT.html#delete/2"},{"type":"function","title":"Parameters - MSSMT.delete/2","doc":"- `tree`: The current tree.\n  - `key`: The key to delete (256-bit binary).","ref":"MSSMT.html#delete/2-parameters"},{"type":"function","title":"Examples - MSSMT.delete/2","doc":"iex> tree = MSSMT.new()\n    iex> key = :crypto.strong_rand_bytes(32)\n    iex> {:ok, tree} = MSSMT.insert(tree, key, \"value\", 100)\n    iex> {:ok, tree} = MSSMT.delete(tree, key)\n    iex> MSSMT.get(tree, key)\n    {:error, :not_found}","ref":"MSSMT.html#delete/2-examples"},{"type":"function","title":"MSSMT.get/2","doc":"Retrieves the value and sum associated with a key in the MSSMT.","ref":"MSSMT.html#get/2"},{"type":"function","title":"Parameters - MSSMT.get/2","doc":"- `tree`: The current tree.\n  - `key`: The key to retrieve (256-bit binary).","ref":"MSSMT.html#get/2-parameters"},{"type":"function","title":"Examples - MSSMT.get/2","doc":"iex> tree = MSSMT.new()\n    iex> key = :crypto.strong_rand_bytes(32)\n    iex> {:ok, tree} = MSSMT.insert(tree, key, \"value\", 100)\n    iex> {:ok, value, sum} = MSSMT.get(tree, key)\n    iex> value\n    \"value\"\n    iex> sum\n    100","ref":"MSSMT.html#get/2-examples"},{"type":"function","title":"MSSMT.insert/4","doc":"Inserts a key-value pair with an associated sum into the MSSMT.","ref":"MSSMT.html#insert/4"},{"type":"function","title":"Parameters - MSSMT.insert/4","doc":"- `tree`: The current tree.\n  - `key`: The key to insert (256-bit binary).\n  - `value`: The value associated with the key.\n  - `sum`: The sum associated with the key.","ref":"MSSMT.html#insert/4-parameters"},{"type":"function","title":"Examples - MSSMT.insert/4","doc":"iex> tree = MSSMT.new()\n    iex> key = :crypto.strong_rand_bytes(32)\n    iex> {:ok, tree} = MSSMT.insert(tree, key, \"value\", 100)\n    iex> {:ok, value, sum} = MSSMT.get(tree, key)\n    iex> value\n    \"value\"\n    iex> sum\n    100","ref":"MSSMT.html#insert/4-examples"},{"type":"function","title":"MSSMT.merkle_proof/2","doc":"Generates a Merkle proof for a given key.","ref":"MSSMT.html#merkle_proof/2"},{"type":"function","title":"Parameters - MSSMT.merkle_proof/2","doc":"- `tree`: The current tree.\n  - `key`: The key to generate the proof for (256-bit binary).","ref":"MSSMT.html#merkle_proof/2-parameters"},{"type":"function","title":"Examples - MSSMT.merkle_proof/2","doc":"iex> tree = MSSMT.new()\n    iex> key = :crypto.strong_rand_bytes(32)\n    iex> {:ok, tree} = MSSMT.insert(tree, key, \"value\", 100)\n    iex> proof = MSSMT.merkle_proof(tree, key)\n    iex> is_list(proof)\n    true","ref":"MSSMT.html#merkle_proof/2-examples"},{"type":"function","title":"MSSMT.new/0","doc":"Creates a new empty MSSMT.","ref":"MSSMT.html#new/0"},{"type":"function","title":"Examples - MSSMT.new/0","doc":"iex> _tree = MSSMT.new()\n    nil","ref":"MSSMT.html#new/0-examples"},{"type":"function","title":"MSSMT.root_hash/1","doc":"Returns the root hash of the MSSMT.","ref":"MSSMT.html#root_hash/1"},{"type":"function","title":"Examples - MSSMT.root_hash/1","doc":"iex> tree = MSSMT.new()\n    iex> root_hash = MSSMT.root_hash(tree)\n    iex> root_hash == <<0::256>>\n    true","ref":"MSSMT.html#root_hash/1-examples"},{"type":"function","title":"MSSMT.total_sum/1","doc":"Returns the total sum of all nodes in the MSSMT.","ref":"MSSMT.html#total_sum/1"},{"type":"function","title":"Examples - MSSMT.total_sum/1","doc":"iex> tree = MSSMT.new()\n    iex> key1 = :crypto.strong_rand_bytes(32)\n    iex> key2 = :crypto.strong_rand_bytes(32)\n    iex> {:ok, tree} = MSSMT.insert(tree, key1, \"value1\", 100)\n    iex> {:ok, tree} = MSSMT.insert(tree, key2, \"value2\", 200)\n    iex> MSSMT.total_sum(tree)\n    300","ref":"MSSMT.html#total_sum/1-examples"},{"type":"function","title":"MSSMT.verify_proof/5","doc":"Verifies a Merkle proof against a root hash.","ref":"MSSMT.html#verify_proof/5"},{"type":"function","title":"Parameters - MSSMT.verify_proof/5","doc":"- `root_hash`: The root hash of the MSSMT.\n  - `key`: The key the proof is for (256-bit binary).\n  - `value`: The value associated with the key.\n  - `sum`: The sum associated with the key.\n  - `proof`: The Merkle proof as a list of sibling nodes.","ref":"MSSMT.html#verify_proof/5-parameters"},{"type":"function","title":"Examples - MSSMT.verify_proof/5","doc":"iex> tree = MSSMT.new()\n    iex> key = :crypto.strong_rand_bytes(32)\n    iex> {:ok, tree} = MSSMT.insert(tree, key, \"value\", 100)\n    iex> proof = MSSMT.merkle_proof(tree, key)\n    iex> root_hash = MSSMT.root_hash(tree)\n    iex> MSSMT.verify_proof(root_hash, key, \"value\", 100, proof)\n    true","ref":"MSSMT.html#verify_proof/5-examples"},{"type":"module","title":"MSSMT.BranchNode","doc":"Represents a branch node in the MSSMT.","ref":"MSSMT.BranchNode.html"},{"type":"function","title":"MSSMT.BranchNode.compute_hash/1","doc":"Computes the hash of the branch node.\n\nThe hash is computed over the concatenation of the left and right child hashes and the sum of their sums (as unsigned little-endian 64-bit integer).","ref":"MSSMT.BranchNode.html#compute_hash/1"},{"type":"function","title":"Examples - MSSMT.BranchNode.compute_hash/1","doc":"iex> MSSMT.BranchNode.compute_hash(branch_node)\n    <<...>>","ref":"MSSMT.BranchNode.html#compute_hash/1-examples"},{"type":"function","title":"MSSMT.BranchNode.new/2","doc":"Creates a new branch node.","ref":"MSSMT.BranchNode.html#new/2"},{"type":"function","title":"Parameters - MSSMT.BranchNode.new/2","doc":"- `left`: The left child node.\n  - `right`: The right child node.","ref":"MSSMT.BranchNode.html#new/2-parameters"},{"type":"function","title":"Examples - MSSMT.BranchNode.new/2","doc":"iex> MSSMT.BranchNode.new(left_node, right_node)\n    %MSSMT.BranchNode{...}","ref":"MSSMT.BranchNode.html#new/2-examples"},{"type":"type","title":"MSSMT.BranchNode.t/0","doc":"","ref":"MSSMT.BranchNode.html#t:t/0"},{"type":"module","title":"MSSMT.LeafNode","doc":"Represents a leaf node in the Merkle Sum Sparse Merkle Tree (MSSMT).","ref":"MSSMT.LeafNode.html"},{"type":"function","title":"MSSMT.LeafNode.compute_hash/1","doc":"Computes the hash of the leaf node.\n\nThe hash is computed over the concatenation of the value and the sum (as unsigned little-endian 64-bit integer).","ref":"MSSMT.LeafNode.html#compute_hash/1"},{"type":"function","title":"Examples - MSSMT.LeafNode.compute_hash/1","doc":"iex> MSSMT.LeafNode.compute_hash(leaf_node)\n    <<...>>","ref":"MSSMT.LeafNode.html#compute_hash/1-examples"},{"type":"function","title":"MSSMT.LeafNode.new/3","doc":"Creates a new leaf node.","ref":"MSSMT.LeafNode.html#new/3"},{"type":"function","title":"Parameters - MSSMT.LeafNode.new/3","doc":"- `key`: The key associated with the leaf (256-bit binary).\n  - `value`: The value stored in the leaf.\n  - `sum`: The sum associated with the leaf.","ref":"MSSMT.LeafNode.html#new/3-parameters"},{"type":"function","title":"Examples - MSSMT.LeafNode.new/3","doc":"iex> MSSMT.LeafNode.new(key, \"value\", 100)\n    %MSSMT.LeafNode{...}","ref":"MSSMT.LeafNode.html#new/3-examples"},{"type":"type","title":"MSSMT.LeafNode.t/0","doc":"","ref":"MSSMT.LeafNode.html#t:t/0"},{"type":"protocol","title":"MSSMT.Node","doc":"Protocol defining the behavior for nodes in the MSSMT.","ref":"MSSMT.Node.html"},{"type":"function","title":"MSSMT.Node.copy/1","doc":"Returns a copy of the node.\n\nSince structs are immutable, this returns the node itself.","ref":"MSSMT.Node.html#copy/1"},{"type":"function","title":"MSSMT.Node.node_hash/1","doc":"Computes the node's hash.","ref":"MSSMT.Node.html#node_hash/1"},{"type":"function","title":"MSSMT.Node.node_sum/1","doc":"Computes the node's sum.","ref":"MSSMT.Node.html#node_sum/1"},{"type":"type","title":"MSSMT.Node.t/0","doc":"All the types that implement this protocol.","ref":"MSSMT.Node.html#t:t/0"},{"type":"module","title":"MSSMT.NodeHash","doc":"Module representing a node hash in the Merkle Sum Sparse Merkle Tree (MSSMT).","ref":"MSSMT.NodeHash.html"},{"type":"function","title":"MSSMT.NodeHash.zero/0","doc":"Returns a zero hash (hash of all zeros).","ref":"MSSMT.NodeHash.html#zero/0"},{"type":"function","title":"Examples - MSSMT.NodeHash.zero/0","doc":"iex> MSSMT.NodeHash.zero()\n    <<0::256>>","ref":"MSSMT.NodeHash.html#zero/0-examples"},{"type":"type","title":"MSSMT.NodeHash.t/0","doc":"","ref":"MSSMT.NodeHash.html#t:t/0"},{"type":"extras","title":"Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"# Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir\n\nA **Merkle-Sum Sparse Merkle Tree (MS-SMT)** is a data structure that combines the features of a Merkle tree and a sum tree, allowing for efficient proofs of inclusion and accumulation of values. It's particularly useful for securely storing large amounts of data with the ability to verify the integrity and sum of the data efficiently.\n\nThis Elixir implementation provides a simple and efficient MS-SMT library for use in your projects.","ref":"readme.html"},{"type":"extras","title":"Table of Contents - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"- [Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir](#merkle-sum-sparse-merkle-tree-ms-smt-in-elixir)\n  - [Table of Contents](#table-of-contents)\n  - [Installation](#installation)\n  - [Usage](#usage)\n    - [Basic Operations](#basic-operations)\n    - [Generating and Verifying Proofs](#generating-and-verifying-proofs)\n  - [API Reference](#api-reference)\n    - [MSSMT.new/0](#mssmtnew0)\n    - [MSSMT.insert/3](#mssmtinsert3)\n    - [MSSMT.get/2](#mssmtget2)\n    - [MSSMT.delete/2](#mssmtdelete2)\n    - [MSSMT.root\\_hash/1](#mssmtroot_hash1)\n    - [MSSMT.total\\_sum/1](#mssmttotal_sum1)\n    - [MSSMT.merkle\\_proof/2](#mssmtmerkle_proof2)\n    - [MSSMT.verify\\_proof/4](#mssmtverify_proof4)\n  - [Development](#development)\n  - [Contributing](#contributing)\n  - [License](#license)","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Installation - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"To use `mssmt` in your Elixir project, add it to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mssmt, git: \"https://github.com/AbdelStark/mssmt.git\", tag: \"v0.1.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Operations - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"```elixir\n# Import the MSSMT module\nimport MSSMT\n\n# Create a new empty MS-SMT\ntree = MSSMT.new()\n\n# Insert key-value pairs\nkey1 = :crypto.strong_rand_bytes(32)\nkey2 = :crypto.strong_rand_bytes(32)\ntree = MSSMT.insert(tree, key1, \"value1\", 100)\ntree = MSSMT.insert(tree, key2, \"value2\", 200)\n\n# Retrieve a value\n{:ok, retrieved_value, retrieved_sum} = MSSMT.get(tree, key)\n\n# Delete a key-value pair\n{:ok, tree} = MSSMT.delete(tree, key1)\n\n# Calculate root hash\nroot_hash = MSSMT.root_hash(tree)  # Returns a 32-byte binary hash\n\n# Calculate total sum\ntotal_sum = MSSMT.total_sum(tree)  # Returns 150\n```","ref":"readme.html#basic-operations"},{"type":"extras","title":"Generating and Verifying Proofs - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"```elixir\nkey1 = :crypto.strong_rand_bytes(32)\nkey2 = :crypto.strong_rand_bytes(32)\nkey3 = :crypto.strong_rand_bytes(32)\ntree = MSSMT.new()\ntree = MSSMT.insert(tree, key1, \"value1\", 100)\ntree = MSSMT.insert(tree, key2, \"value2\", 200)\ntree = MSSMT.insert(tree, key3, \"value3\", 300)\n\nproof = MSSMT.generate_proof(tree, key1)\n\nroot_hash = MSSMT.root_hash(tree)\n\n# Verify the proof\nis_valid = MSSMT.verify_proof(root_hash, key1, \"value1\", proof)  # Returns true\n\n# Verify with incorrect value\nis_valid = MSSMT.verify_proof(root_hash, key1, \"value2\", proof)  # Returns false\n\n# Verify with incorrect key\nis_valid = MSSMT.verify_proof(root_hash, key2, \"value2\", proof)  # Returns false\n```","ref":"readme.html#generating-and-verifying-proofs"},{"type":"extras","title":"API Reference - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"","ref":"readme.html#api-reference"},{"type":"extras","title":"MSSMT.new/0 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Creates a new empty Merkle-Sum Sparse Merkle Tree.\n\n**Example:**\n\n```elixir\ntree = MSSMT.new()\n```\n\n---","ref":"readme.html#mssmt-new-0"},{"type":"extras","title":"MSSMT.insert/3 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Inserts a key-value pair into the tree.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n  - `value`: A numeric value associated with the key.\n  - `sum`: The sum of the value.\n\n**Example:**\n\n```elixir\nkey = :crypto.strong_rand_bytes(32)\n{:ok, tree} = MSSMT.insert(tree, key, \"value1\", 100)\n```\n\n---","ref":"readme.html#mssmt-insert-3"},{"type":"extras","title":"MSSMT.get/2 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Retrieves the value associated with a key.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n\n**Returns:** The value and sum associated with the key, or `{:error, :not_found}` if the key does not exist.\n\n**Example:**\n\n```elixir\n{:ok, retrieved_value, retrieved_sum} = MSSMT.get(tree, key)\n```\n\n---","ref":"readme.html#mssmt-get-2"},{"type":"extras","title":"MSSMT.delete/2 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Deletes a key-value pair from the tree.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n\n**Example:**\n\n```elixir\nkey = :crypto.strong_rand_bytes(32)\n{:ok, tree} = MSSMT.delete(tree, key)\n```\n\n---","ref":"readme.html#mssmt-delete-2"},{"type":"extras","title":"MSSMT.root_hash/1 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Calculates the root hash of the tree.\n\n- **Parameters:**\n  - `tree`: The current tree.\n\n**Returns:** A 32-byte binary hash representing the root of the tree, or `nil` if the tree is empty.\n\n**Example:**\n\n```elixir\nroot_hash = MSSMT.root_hash(tree)\n```\n\n---","ref":"readme.html#mssmt-root_hash-1"},{"type":"extras","title":"MSSMT.total_sum/1 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Calculates the total sum of all values in the tree.\n\n- **Parameters:**\n  - `tree`: The current tree.\n\n**Returns:** The total sum as a number.\n\n**Example:**\n\n```elixir\ntotal_sum = MSSMT.total_sum(tree)\n```\n\n---","ref":"readme.html#mssmt-total_sum-1"},{"type":"extras","title":"MSSMT.merkle_proof/2 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Generates a proof of inclusion for a given key.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n\n**Returns:** A list of nodes required to verify the inclusion of the key.\n\n**Example:**\n\n```elixir\nkey = :crypto.strong_rand_bytes(32)\nproof = MSSMT.merkle_proof(tree, key)\n```\n\n---","ref":"readme.html#mssmt-merkle_proof-2"},{"type":"extras","title":"MSSMT.verify_proof/4 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Verifies a proof of inclusion for a given key and value.\n\n- **Parameters:**\n  - `root_hash`: The root hash of the tree.\n  - `key`: A binary representing the key.\n  - `value`: The numeric value associated with the key.\n  - `proof`: The proof generated by `generate_proof/2`.\n\n**Returns:** `true` if the proof is valid, `false` otherwise.\n\n**Example:**\n\n```elixir\nkey = :crypto.strong_rand_bytes(32)\nis_valid = MSSMT.verify_proof(root_hash, key, \"value1\", 100, proof)\n```","ref":"readme.html#mssmt-verify_proof-4"},{"type":"extras","title":"Development - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"To set up the project for development:\n\n1. **Clone the repository:**\n\n   ```bash\n   git clone https://github.com/AbdelStark/mssmt.git\n   cd mssmt\n   ```\n\n2. **Install dependencies:**\n\n   ```bash\n   mix deps.get\n   ```\n\n3. **Run tests:**\n\n   ```bash\n   mix test\n   ```","ref":"readme.html#development"},{"type":"extras","title":"Contributing - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Contributions are welcome! Please feel free to submit a Pull Request.\n\nWhen contributing, please:\n\n- Write clear, concise commit messages.\n- Write tests for new functionality.\n- Ensure all tests pass before submitting.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2024 @AbdelStark\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}